#define BOOST_TEST_DYN_LINK
#define BOOST_TEST_MODULE 0007
#define BOOST_TEST_NO_MAIN

#include <boost/test/unit_test.hpp>
#include <AMDiS.h>

using namespace AMDiS;
using namespace std;

/*
  - Test must run with 4 ranks.

  - Test for function Triangle::getSubBoundary()
  
  - Test on edge boundary between rank 0/1 and 2/3
*/

BOOST_AUTO_TEST_CASE(amdis_mpi_triangle_getsubboundary)
{
  BOOST_REQUIRE(MPI::COMM_WORLD.Get_size() == 4);

  ProblemStat ellipt("ellipt");
  ellipt.initialize(INIT_ALL);
  MeshDistributor::globalMeshDistributor->initParallelization();

  Mesh* mesh = ellipt.getMesh();

  BOOST_REQUIRE(mesh->getNumberOfElements() == 2047);
  BOOST_REQUIRE(mesh->getNumberOfLeaves() == 1024);

  MeshDistributor &meshDist = *(MeshDistributor::globalMeshDistributor);
  InteriorBoundary &intBoundary = meshDist.getIntBoundary();

  vector<AtomicBoundary> boundary;
  switch (MPI::COMM_WORLD.Get_rank()) {
  case 0:
    boundary = intBoundary.getOther(1);
    break;
  case 1:
    boundary = intBoundary.getOwn(0);
    break;
  case 2:
    boundary = intBoundary.getOther(3);
    break;
  case 3:
    boundary = intBoundary.getOwn(2);
    break;
  default:
    BOOST_ERROR("Wrong rank number!\n");
  }

  BOOST_REQUIRE(boundary.size() > 0);

  
  bool found = false;
  vector<BoundaryObject> subBound;

  for (unsigned int i = 0; i < boundary.size(); i++) {
    if (boundary[i].rankObj.subObj == EDGE) {
      boundary[i].rankObj.el->getSubBoundary(boundary[i].rankObj, subBound);
      found = true;
    }
  }

  BOOST_CHECK(found);
  BOOST_CHECK(subBound.size() == 32);

  Spreadsheet sheet;
  sheet.read("data/data0007/bound_rank_" + lexical_cast<string>(MPI::COMM_WORLD.Get_rank()) + ".dat");
  vector<vector<double> > &data = sheet.getData();
}


int main(int argc, char **argv)
{
  AMDiS::init(argc, argv, "./init/test0007.dat.2d");

  boost::unit_test::unit_test_main(&init_unit_test, argc, argv);

  AMDiS::finalize();
}
